#! /usr/bin/python3

with open('pyproject.toml') as f:
    updated_content = f.read().replace(
        'python_version = "3.13"',
        'python_version = "3.14"'
    )
with open('pyproject.toml', 'w') as f:
    f.write(updated_content)



from enum import Enum

ACCESS_TOKEN_VAR_NAMES = {
    'gitlab.com': 'GITLAB_ACCESS_TOKEN',
    'github.com': 'GITHUB_OAUTH_TOKEN',
    'DEFAULT': 'GIT_TOKEN',
}


class CloningStates(Enum):
    NOT_STARTED = 'NOT_STARTED'
    SOURCE_BRANCH_DOES_NOT_EXIST = 'SOURCE_BRANCH_DOES_NOT_EXIST'
    ACCESS_TOKEN_NOT_PROVIDED = 'ACCESS_TOKEN_NOT_PROVIDED'
    CLONING = 'CLONING'
    CLONED = 'CLONED'
    NOT_FOUND = 'NOT_FOUND'
    SKIPPED = 'SKIPPED'


class PullRequestStates(Enum):
    NOT_CREATED = 'NOT_CREATED'
    CREATED = 'CREATED'
    GOT_BAD_RESPONSE = 'GOT_BAD_RESPONSE'
    MERGED = 'MERGED'


class ModificationState(Enum):
    NOT_STARTED = 'NOT_STARTED'
    GOT_EXCEPTION = 'GOT_EXCEPTION'
    MODIFIED = 'MODIFIED'
    NO_FILES_CHANGED = 'NO_FILES_CHANGED'
    PUSHED_TO_REMOTE = 'PUSHED_TO_REMOTE'



import os
from random import randint
from string import ascii_letters, digits
from urllib.parse import urlparse

from git.cmd import Git

from autogit.constants import ACCESS_TOKEN_VAR_NAMES
from autogit.data_types import RepoState


def remove_suffix(value: str, suffix: str, case_insensitive: bool = True) -> str:
    """Removes suffix from a given string `value` case_insensitive by default."""
    if value.endswith(suffix) or (case_insensitive and value.lower().endswith(suffix.lower())):
        return value[: -len(suffix)]
    return value


def get_random_hex() -> str:
    """Returns 8 digit hex code."""
    return hex(randint(2**31 + 1, 2**32))[2:]


def to_kebab_case(value: str) -> str:
    """Converts provided text `value` to a kebab-case name with max length of 100 for a branch name."""
    allowed_chars = ascii_letters + digits + ' -/'
    value = (
        value.replace('.', ' ')
        .replace(',', ' ')
        .replace('\\', ' ')
        .replace(':', ' ')
        .replace(';', ' ')
    )
    ascii_value = ''.join([char for char in value if char in allowed_chars])
    return '-'.join(ascii_value.lower().split())[:100]


def get_access_token(url: str) -> str:
    """Retrieves access token from environment vars suitable for provided URL."""
    parsed_url = urlparse(url)
    domain = parsed_url.netloc.split('@')[-1].lower()
    access_token_var_name = ACCESS_TOKEN_VAR_NAMES.get(domain, ACCESS_TOKEN_VAR_NAMES['DEFAULT'])
    return os.getenv(access_token_var_name, '')


def get_domain(url: str) -> str:
    """Returs domain name in lower case."""
    parsed_url = urlparse(url)
    return parsed_url.netloc.split('@')[-1].lower()


def get_repo_owner(url: str) -> str:
    """Parses repository owner from a valid Git url."""
    return url.rsplit('/', 2)[-2]


def get_repo_path(url: str) -> str | None:
    parsed_url = urlparse(url)
    return parsed_url.path.lstrip('/').removesuffix('.git')


def get_repo_name(url: str) -> str:
    """Parses repository name from a valid Git url."""
    return remove_suffix(url.split('/')[-1], '.git')


def get_default_branch(repo: RepoState):
    g = Git(repo.directory)
    default_branch_name: str = g.execute(['git', 'rev-parse', '--abbrev-ref', 'origin/HEAD'])  # type: ignore
    return default_branch_name.split('/', 1)[-1]  # removes `origin/` prefix from the result



"""Action:
- Parse domain names from provided list of repositories.
- Detect known Access Tokens from environment variables.
- Detect for each domain if it is Github or Gitlab and API version.
- Detect which Access Token works with which domain.


Returns:
 domains = {
   "<domain_name>": {
        "token": "<token>",
        "api_version": "<version>",
        "type": "Gitlab|Github",
    }
 }

"""

from autogit.data_types import RepoState


def get_tokens_and_api_versions(repos: dict[str, RepoState]) -> None:
    """:return: domains = {
      "<domain_name>": {
        "token": "<token>",
        "api_version": "<version>",
        "type": "Gitlab|Github",
      }
    }
    """
    # TODO: implement this part.

    ## TODO: add support for such API keys in environment:
    # 'GITLAB_ACCESS_TOKEN': '<GITLAB_ACCESS_TOKEN>',
    # 'GITLAB_OAUTH_TOKEN': '<GITLAB_ACCESS_TOKEN>',
    # 'GITLAB_TOKEN': '<GITLAB_ACCESS_TOKEN>',
    #
    # 'GITHUB_OAUTH_TOKEN': '<GITHUB_OAUTH_TOKEN>',
    # 'GITHUB_TOKEN': '<GITHUB_OAUTH_TOKEN>',
    # 'GITHUB_ACCESS_TOKEN': '<GITHUB_OAUTH_TOKEN>',
    #
    # 'GIT_TOKEN': '<GIT_TOKEN>',
    # 'GIT_ACCESS_TOKEN': '<GIT_TOKEN>',
    # 'GIT_OAUTH_TOKEN': '<GIT_TOKEN>',


# curl --header "PRIVATE-TOKEN: personal-access-token" your-gitlab-url/api/v4/version



from autogit.actions._1_parse_arguments import parse_command_line_arguments
from autogit.actions._2_get_repository_states import get_repository_states
from autogit.actions._4_clone_repositories import clone_repositories
from autogit.actions._5_create_branch import create_branch_for_each_repo
from autogit.actions._6_run_command import run_command_for_each_repo
from autogit.actions._7_commit_and_push_changes import (
    commit_and_push_changes_for_each_repo,
)
from autogit.actions._8_create_pull_request import create_pull_request_for_each_repo
from autogit.utils.throttled_tasks_executor import ThrottledTasksExecutor


def main(args: list[str] | None = None) -> None:
    cli_args = parse_command_line_arguments(args)
    repos = get_repository_states(cli_args)

    with ThrottledTasksExecutor(delay_between_tasks=0.1) as executor:
        clone_repositories(repos, executor)
        create_branch_for_each_repo(repos, executor)
        run_command_for_each_repo(repos, executor)
        commit_and_push_changes_for_each_repo(repos, executor)
        create_pull_request_for_each_repo(repos, executor)


if __name__ == '__main__':
    main()



import argparse
import logging
import sys

from autogit.data_types import CliArguments
from autogit.utils.helpers import get_random_hex


def get_argument_parser() -> argparse.ArgumentParser:
    action_id = get_random_hex()

    parser = argparse.ArgumentParser(
        description='Update multiple GitLab or GitHub repositories with a single command.',
        epilog="""Report bugs and request features at https://github.com/albertas/auto-git/issues""",
        add_help=False,
    )

    parser.add_argument(
        '-r',
        '--repos',
        action='append',
        dest='repos',
        nargs='?',
        default=[],
        type=str,
        help='Repository url or Path to a file containing list of repository urls',
    )

    parser.add_argument(
        '-c',
        '--clone-to',
        action='store',
        dest='clone_to',
        default='/tmp/',
        type=str,
        nargs='?',
        help='Path to directory which will be used to clone git repositories to (default is /tmp/)',
    )

    parser.add_argument(
        '-m',
        '--message',
        '--commit-message',
        action='store',
        dest='commit_message',
        default=f'AutoGit action #{action_id}',
        type=str,
        nargs='?',
        help='Message which will be used for commit message (and for branch name, PR title if they are not specified)',
    )

    parser.add_argument(
        '-v',
        '--verbose',
        action='store_const',
        dest='verbose',
        default=False,
        const=True,
        help='Increase verbosity and show DEBUG logs',
    )

    parser.add_argument(
        '-s',
        '--source-branch',
        action='store',
        dest='source_branch',
        type=str,
        nargs='?',
        help='Base branch which will be used as a basis for a new branch creation. Target branch will be used by default.',
    )

    parser.add_argument(
        '-b',
        '--branch',
        action='store',
        dest='branch',
        type=str,
        nargs='?',
        help='A name of a branch which will be used to commit changes to and to create Pull Request from (this branch will be created if it does not exist)',
    )

    parser.add_argument(
        '-t',
        '--target-branch',
        action='store',
        dest='target_branch',
        type=str,
        nargs='?',
        help='Branch to be used as a target in a Pull Request.',
    )

    parser.add_argument(
        '-h',
        '--help',
        action='help',
        default=argparse.SUPPRESS,
        help='Show this message and exit.',
    )

    parser.add_argument(
        'commands',
        type=str,
        nargs='*',
        help='Commands or scripts to execute in local Git repository',
    )

    parser.add_argument(
        '--action-id',
        action='store_const',
        dest='action_id',
        const=action_id,
        help='Unique hex for action identification',
    )

    return parser


def parse_command_line_arguments(args: list[str] | None = None) -> CliArguments:
    if args is None:
        args = sys.argv[1:]

    parser = get_argument_parser()
    parsed_args: CliArguments = parser.parse_args(args=args)  # type: ignore

    if parsed_args.verbose:
        logging.getLogger().setLevel(logging.DEBUG)

    return parsed_args



import logging
import os
import os.path

from autogit.data_types import CliArguments, RepoState
from autogit.utils.helpers import (
    get_domain,
    get_repo_path,
    get_repo_name,
    get_repo_owner,
    to_kebab_case,
)

logger = logging.getLogger()


def is_url_or_git(file_names_or_repo_url: str) -> bool:
    # TODO: use urlparse to verify if its url and use regexp for git url
    return '.com' in file_names_or_repo_url.lower()


def read_repositories_from_file(repos_filename: str) -> list[str]:
    """Reads a list of repositories from a file while ignoring commented out lines."""
    with open(repos_filename) as f:
        return [line.strip() for line in f if not line.strip().startswith('#')]


def get_repository_state(
    repo_url: str,
    args: CliArguments = None,
) -> RepoState:
    repo_name = get_repo_name(repo_url)
    repo_owner = get_repo_owner(repo_url)
    repo_path = get_repo_path(repo_url)
    domain = get_domain(repo_url)

    return RepoState(
        args=args,
        name=repo_name,
        owner=repo_owner,
        path=repo_path,
        url=repo_url,
        domain=domain,
        source_branch=args.source_branch,
        branch=args.branch,
        target_branch=args.target_branch,
    )


def get_repository_states(args: CliArguments) -> dict[str, RepoState]:
    repo_urls = []
    for file_names_or_repo_url in args.repos:
        if not is_url_or_git(file_names_or_repo_url) and os.path.exists(file_names_or_repo_url):
            newly_read_repos = read_repositories_from_file(file_names_or_repo_url)
            repo_urls.extend(newly_read_repos)
        else:
            repo_urls.append(file_names_or_repo_url)

    if not args.branch:
        args.branch = to_kebab_case(args.commit_message)
        ## TODO: print this when verbosity is turned on
        # print(
        #     f'\033[1;32m|\033[0m {(f"New branch:  {args.branch}").ljust(75, " ")} \033[1;32m|\033[0m\n'
        # )

    repos: dict[str, RepoState] = {}
    for repo_url in repo_urls:
        repo_state = get_repository_state(
            repo_url=repo_url,
            args=args,
        )
        repos[repo_state.name] = repo_state

    return repos
